/* Main Source for Servo Tester
*/

#include "int18XXX.H"
#pragma unlockISR
#pragma sharedAllocation
#include "TServos.h"

/** V E C T O R  R E M A P P I N G *******************************************/

#pragma origin ResetVector		// specific PIC bootloader@ org 0

void main( void);

void dummy (void)
{
#asm
call main
#endasm

}

/* ---------------------------------------------------- */
/*              INTERRUPTS HANDLER                      */
/* ---------------------------------------------------- */

#pragma origin ResetVector + 8
/******************* INTERRUPTS ********************************
Specific PICdem with bootloader@ org 0
compiler refuse this address, so normal procedure, not interrupt.
interrupt High_int_server (void) is normal procedure when 0x08 
these procedures need a dummy call at init to have variables allocated.
Uncomment registers to save as needed.
**************************************************************/

/********* HIGH PRIORITY INTERRUPT **************/
interrupt High_int_server( void)
{
	High_Int();
	#pragma fastMode
}	
		

/**********LOW PRIORITY INTERRUPT **************/	

#pragma origin ResetVector + 0x18

interrupt Low_Int_Server(void)
{
	int_save_registers

// ****** real time 1 ms clock

	if (TMR0IF)	
	{
		// real time clock every millisecond
		TMR0H = TIMER0H;	//0xd1;
		TMR0L = TIMER0L;	//0x20;
		TMR0IF = 0;
		ms = ms + 1;
		msflg = 1;
		if (ms == 1000)
		{
			sec = sec + 1;
			secflg = 1;
			//Led = !Led;
			ms = 0;
			if (sec == 60)
			{
				min = min + 1;
				sec = 0;
				
				if (min == 60)
				{
					hour = hour + 1;
					min = 0;
				}
			}
		}
		if (timer1 == 1)
		{
			tempoms--;
			if (tempoms == 0)
			{
				timer1 = 0;
			}
		}
		if (RotaryDebounce != 0)
		{
			RotaryDebounce--;
		}		
	}
// int on chg for rotary
	if (IOCIF)
	{
		CpyPB = PORTB;
		HandelRotary();	
		IOCIF = 0;	
	}
// ***** Collect analog ports values
	if (ADIF)		//Get the analog values 
	{

		ADIF = 0;
		ADRes.high8 = ADRESH;
		ADRes.low8 = ADRESL;
		switch (NextANP)
		{
			case 0:
			{
				ANP0 = ADRes;
				if (ANP0 < 2) ANP0 = 0;
				break;
			}
			case 1:
			{
				ANP1 = ADRes;
				break;
			}		
			case 2:
			{
				ANP2 = ADRes;
				break;
			}
			case 3:
			{
				ANP3 = ADRes;
				break;
			}	
		}	
	}
/******* Voltage control by PWM ******
DAC gives a reference, value is fed to the 2 comparators and 
compared to servo voltage (pin 2), then pwn increased or 
decreased as needed
*************************************************/
	if (TMR2IF == 1)
	{
		TMR2IF = 0;
		if (C2OUT == 1)
		{
			if (PWMDuty < 1022)
			{
				PWMDuty++;
				PWMDuty++;
			}	
		}
		if (C1OUT == 1)
		{
			if (PWMDuty > 1)
			{
				 PWMDuty--;
				 PWMDuty--;
			}
		}
		intdat = PWMDuty/4;
		CCPR1L = intdat.low8;
		CCP1CON.4 = PWMDuty.0;
		CCP1CON.5 = PWMDuty.1;	
			
	}	

	int_restore_registers
}

/**************HIGH PRIORITY ROUTINES **************/
void High_Int (void)
{
	uns8 sv_PRODL = PRODL;
	uns8 sv_PRODH = PRODH;
	// TMR2 generate pulses	
	if (TMR1IF == 1)
	{
		TMR1IF = 0;
		if (TimingMode == 0)
		{
			if (!StartStp)
			{
			CPANP0 = ANP0 / 2;
			CPANP0 += 256;
			if (CPANP0 > 511) 	
			{
				CpCalc = CPANP0 - 511;
				CpCalc *= LimiteH;
				CpCalc /= 511;
				ModANP0 = CpCalc; 
				Dir = 1;
			}
			else
			{
				CpCalc = 512 - CPANP0;
				CpCalc *= LimiteL;
				CpCalc /= 512;
				ModANP0 = CpCalc;
				Dir = 0;
			}		
			ANcalc = Pos0;	//1,5 ms base
			ANcalc2 = ModANP0 * 8;
			if (Dir) 
			{
				ANcalc = ANcalc - ANcalc2;
			}
			else 
			{
				ANcalc = ANcalc + ANcalc2;
			}
			}
			else
			{
				if (SW1 == 1) ANcalc = 2400;
				else ANcalc = 6000;
			}	
			if (ServoSeq == 0)
			{		
				ANcalc2 = 65535 - ANcalc;
				TMR1H = ANcalc2.high8;
				TMR1L = ANcalc2.low8;
				mest = ANcalc2;
				Tmr1EndFg = 1;
				LATD = 0xFF;
				ServoSeq = 1;
			}
			else if (ServoSeq == 1)	
			{
				TMR1H = 0x15;	//65535-60000 >> 20ms pulse
				TMR1L = 0x9F;
				LATD = 0;
				ServoSeq = 0;
			}
		}	 
		else if (TimingMode == 1)
		{
			if (ServoSeq == 0)
			{		
				ANcalc2 = 65535 - ANcalc;
				TMR1H = ANcalc2.high8;
				TMR1L = ANcalc2.low8;
				mest = ANcalc2;
				Tmr1EndFg = 1;
				LATD = 0xFF;
				ServoSeq = 1;
			}
			else if (ServoSeq == 1)	
			{
				TMR1H = 0x15;	//65535-60000 >> 20ms pulse
				TMR1L = 0x9F;
				LATD = 0;
				ServoSeq = 0;
			}
		}

	}
	PRODL = sv_PRODL;
	PRODH = sv_PRODH;
}

#include "TservosHrdwrUtils.c" 
#include "LCD.c"
#include "Menus.c"
#include "Ops.c"
/*
void LcdUpdate (bit tmrfg)
{
	
	char i;
	
	if (tmrfg == 1)
	{
		LcdTmr ++;
		if (LcdTmr == 1000) LcdTmr = 0;
	}
	if (LcdTmr == 0)
	{
		cvhexbcdN (upcntr);
		writepos (mil,0,0);
		outdata (cent);
		outdata (dix);
		outdata (un);
		outdata (' ');
	}
	writepos (OutBuf[0],0,1);
	for (i = 1;i!=16;i++)
	{
		outdata (OutBuf[i]);
	}		
}
*/
void HouseKeeping (bit ms)
{

	if (ms)
	{
		ScanAnalog();
	}	 
}		
void InitHard (void)
{
// init clocking system for internal clock
	OSCTUNE = 0x80; //3X PLL ratio mode selected
	OSCCON = 0x70;  //Switch to 16MHz HFINTOSC
	OSCCON2 = 0x10; //Enable PLL, SOSC, PRI OSC drivers turned off
	while(PLLRDY != 1);   //Wait for PLL lock
	ACTCON = 0;  //Enable active clock tuning for USB operation
// all ports as digital at reset
	ANSELA = 0x0F;	//4 analog ports
	ANSELB = 0;	
	ANSELC = 0;
	ANSELD = 0;
	ANSELE = 0;
	LATA = 0;
	LATB = 0;
	LATC = 0;
	LATD = 0;
	LATE = 0;
	TRISA = 0x0F;
	TRISB = 0xFF;
	TRISC = 0xFF;
	TRISD = 0;
	TRISE = 0;
	TRISC.1 = 0;
// turn power off on all periph, must be indivually turned on
	PMD0 = 0xEF;	//ACTMD on
	PMD1 = 0xFF;
	TBLPTRU = 0;
	WDTCON = 0;
	RBPU = 0;
// init intrpt
	INTCON = 0;
	INTCON2 = 0;
	INTCON3 = 0;
	PIR1 = 0;
	PIR2 = 0;
	PIE1 = 0;	
	PIE2 = 0;
	IPEN = 1;
//init tmr0, real time clock
	T0CON = bin(00001000);	// 16 bits tmr, no presc  
	TMR0H = 250;
	TMR0L = 1;
 	TMR0IP = 0;	// low prior
 	TMR0ON = 1;
	TMR0IE = 1;
// init intrpt on change for rotary selector
	IOCIP = 0;				//low priority
	IOCB = 0b00110000;		// select inputs 
	IOCC = 0;
	IOCIE = 1;
//init PWM
	TMR2MD = 0;
	CCP1MD = 0;
	nop();
	nop();
	TRISC.2 = 0;
	CCPR1L = 0;
	TMR2IE = 0;
	TMR2IP = 0;
	PR2 = 255;	
	T2CON = 0b00000011;	// prescale 16, tmr is off
//	T2CON = 0b00000001;	// prescale 4, TMR off
	CCP1CON = 0b00001100;
//init Timer 1
	TMR1MD = 0;			//enable tmr hardw
	nop();
	nop();
	T1CON = 0b00100010;  //instr clk, presc4, 16bits writes,off
	// instr clk = 12mhz > base period = 0.3333 usec > 1ms=3000
	T1GCON = 0b00000000;	//not used
	TMR1IE = 0;			//no int yet
	TMR1IP = 1;
// Init Comparator
	CMP1MD = 0;	// pwr on
	CMP2MD = 0;	// pwr on
	nop();
	nop();
	CM1CON0 = 0b10011101; 	//Polarity inv	
	CM2CON0 = 0b10001101;	//Polarity nml
	CM2CON1 = 0b00000000;	//DAC input
// Init DAC
	VREFCON1 = 0b11000000;
	VREFCON2 = 0x11;
// init Analog ports
	ADCMD = 0;		// pwr on
	nop();
	nop();
	ADCON1 = 0b00000000;
	ADCON2 = 0b10100110;
	ADCON0 = 0b00000001;
	ADIP = 0;
	ADIF = 0;
	ADIE = 1;			// results collected via interrupt		// results collected via interrupt
// Start Interrupts
	GIEH = 1;
	GIEL = 1;	
}

void InitVar (void)
{
	clearRAM();
	tempo(2,1);
	CoefVolt.high8 = readprom(0);
	tempo(2,1);
	CoefVolt.low8 = readprom(1);
	CoefAmp.high8 = readprom(2);
	CoefAmp.low8 = readprom(3);
	MenuMode = 1;
	Volt = 551;
	Pos0 = 4500;
	LimiteH = 511;
	LimiteL = 512;
	TimingMode = 0;
}	
	
void main (void)
{
	bit Ms;
	char i;
	
	InitHard();
	InitVar();
	lcdinit();
	InitRotary();
	cleardisplay();
	writepos ('R',0,0);
	outdata ('e');
	outdata ('a');
	outdata ('d');
	outdata ('y');
	tempo(500,1);
	MenuInit ();
	while (1)
	{
		if (msflg == 1)
		{
			Ms = 1;
			msflg = 0;
		}
		HouseKeeping (Ms);
		if (MenuMode) Menu(Ms);
		else Ops(Ms);
		Ms = 0;
		secflg = 0;
	}
}	